@@ -720,14 +720,17 @@ return 0;
 void use_compteur(branche *branches, int count)	
 {
 int num;
-char text[20];
+char text[20], *p;
 branche *br;
+char nan[] = "NaN";
 
+if (count) p = text;
+else p = nan;
 for(num = 0; num < totbranches; num++) {
 	br = branches + num;
 	if(br->br_label != NULL) free(br->br_label);
-	sprintf(text, "%d", (int)(br->compteur * 100. / count + 0.5) );
-	br->br_label = strdup(text);
+	if (count) sprintf(text, "%d", (int)(br->compteur * 100. / count + 0.5) );
+	br->br_label = strdup(p);
 	}
 }
 
