@@ -54,8 +54,7 @@ extern int set_res_value(const char *name, const char *value);
 extern void delete_res_value(const char *name);
 extern void save_resources(void);
 extern char *create_tmp_filename(void);
-extern int save_fasta_file(const char *fname, char **seq, char **comments, char **seqname, 
-	int totseqs, int *eachlength, region *region_used, int *sel_seqs, int tot_sel_seqs, int spaces_in_names);
+extern void delete_tmp_filename(const char *base_fname);
 extern void direct_help_callback(Fl_Widget *wgt, void *data);
 extern int compute_size_params(SEA_VIEW *view, int force_recompute);
 extern char **prepcolranks_by_codon(char **seq, int totseqs, int maxlen, int *eachlength, char **comments);
@@ -191,6 +190,7 @@ status = save_part_as_pir(debut, fin, view->sequence, view->seqname,
 if(status) {
 	fl_alert("Cannot write sequences to filename\n%s", fname);
 	free(tmpselseqs);
+	delete_tmp_filename(base_fname);
 	return;
 	}
 if( my_align_choice == ALIGN_SITES ) num_longest = confirm_refer_seq(num_longest, view, tmpselseqs);
@@ -198,6 +198,7 @@ if(num_longest == -1 ) {
 	sprintf(fname, "%s.pir", base_fname);
 	remove(fname);
 	free(tmpselseqs);
+	delete_tmp_filename(base_fname);
 	return;
 	}
 
@@ -205,6 +206,7 @@ commande = calc_msa_command(view, base_fname);
 status = (commande == NULL);
 if(status) {
 	free(tmpselseqs);
+	delete_tmp_filename(base_fname);
 	return;
 	}
 status = run_external_prog_in_pseudoterm(commande, NULL, "alignment");
@@ -231,12 +233,7 @@ if (is_view_valid(view)) {
 	  }
 }
 free(tmpselseqs);
-#ifdef WIN32
-  sprintf(fname, "DEL %s.*", base_fname);
-#else
-  sprintf(fname, "PATH=/bin:/usr/bin rm %s.*", base_fname);
-#endif
-system(fname);
+delete_tmp_filename(base_fname);
 }
 
 
@@ -694,8 +691,7 @@ void cre_align_menu(SEA_VIEW *view)
       alignitems[clustalopt + elt - 1].label(strdup(name));
       if (elt == view->count_msa_algos) attr |= FL_MENU_DIVIDER;
 #ifndef WIN32
-      if(get_full_path(name) == NULL) {
-	attr = (alignitems + clustalopt + elt - 1)->flags;
+      if (get_full_path(name) == NULL) {
 	attr |= FL_MENU_INACTIVE;
 	} 
 #endif
@@ -873,7 +869,8 @@ if(w == NULL) {
 	args = new Fl_Input(85, 40, 430, 20, "Arguments");
 	args->tooltip(
 		"probcons example: %f.pir   >   %f.out\n"
-		"t_coffee example: %f.pir -outfile=%f.out -output=fasta_aln -outorder=input"
+		"t_coffee example: %f.pir -outfile=%f.out -output=fasta_aln\n"
+		"mafft example: --auto %f.pir  >  %f.out"
 	);
 	chooser = new Fl_Button(345, 10, 160, 20, "Select external program");
 	chooser->callback(choose_align_callback, (void *)name);
@@ -1079,6 +1076,7 @@ void profile_align(SEA_VIEW *view)
 		total++;
 		if(view->sel_seqs[num]) {
 			fl_alert("Selected sequences should not belong to profile sequences");
+		  delete_tmp_filename(base_fname);
 			return;
 			}
 		}
@@ -1087,7 +1085,7 @@ void profile_align(SEA_VIEW *view)
 							 view->list_species_sets[num_sp_set]);
 	if(status) {
 		fl_alert("Cannot write sequences to filename\n%s", fname1);
-		remove(fname1);
+	  delete_tmp_filename(base_fname);
 		return;
 		}
 	replace_stars(fname1);
@@ -1102,7 +1100,7 @@ void profile_align(SEA_VIEW *view)
 	replace_stars(fname2);
 	if(status) {
 		fl_alert("Cannot write sequences to filename\n%s", fname2);
-		remove(fname2);
+	  delete_tmp_filename(base_fname);
 		return;
 		}
 	p = get_res_value( profile_algo == profile_clustalo ? "msa_name_1" : "msa_name_2", NULL);
@@ -1167,7 +1165,7 @@ void profile_align(SEA_VIEW *view)
 			if(num > 0) free(seq); 
 		}
 	}
-	remove(fname1);
+  delete_tmp_filename(base_fname);
 	if ( !is_view_valid(view) ) return;
 	if( status != 0 || num != total) {
 		fl_message("%s: alignment operation ignored", extract_filename(view->masename));
