@@ -171,7 +171,7 @@ for(num = 0; num < view->tot_comment_lines; num++) {
 	total = numgaps;
 	if(total + view->comment_length[num] > view->max_seq_length) {
 		total = view->max_seq_length - view->comment_length[num];
-		fl_ringbell(0);
+		fl_beep(FL_BEEP_DEFAULT);
 		}
 	debut = view->comment_line[num] + pos - 1;
 	l = view->comment_length[num] - pos + 1;
