@@ -20,6 +20,8 @@
 #include <math.h>
 #include <ctype.h>
 #include <limits.h>
+#include <setjmp.h>
+static jmp_buf lngjmp_env;
 
 #define Malloc(x) mymalloc(x)
 
@@ -35,6 +37,8 @@ typedef unsigned char boolean;
 
 static void    *mymalloc(size_t x);
 
+extern int tree_build_interrupted;
+
 int nmlngth;                /* max number of characters in species name    */
 #define maxuser         2   /* maximum number of user-defined trees */
 
@@ -85,7 +89,7 @@ short nextree, which, minwhich;
 double like_d, minsteps_d, bestyet_d, bestlike_d, bstlike2_d;
 boolean lastrearr_d, recompute_d;
 double nsteps[maxuser];
-short **fsteps;
+static short **fsteps;
 node *there_d;
 short *place_d;
 short **bestrees;
@@ -518,6 +522,9 @@ Local Void evaluate(node* r)
   long term;
   double sum;
 
+  if (tree_build_interrupted) {
+    longjmp(lngjmp_env, 0);
+    }
   sum = 0.0;
   for (i = 0; i < endsite; i++) {
     steps = r->numsteps[i];
@@ -1001,6 +1008,7 @@ Static char *maketree(void)
   temp1->numsteps = (stepshortptr)Malloc(endsite*sizeof(short));
   temp1->base = (baseptr)Malloc(endsite*sizeof(short));
 
+  if (setjmp(lngjmp_env)) goto way_out;
 
   if (!usertree) {
     recompute_d = true;
@@ -1078,22 +1086,25 @@ Static char *maketree(void)
     for (j = 1; j <= maxuser; j++)
       fsteps[j - 1] = (short *)Malloc(endsite*sizeof(short));
     which = 1;
-	free(treefile); treefile = NULL;
+    free(treefile); treefile = NULL;
     if( treeread() == 0) {
 		postorder(root);
 		evaluate(root);
 		treefile = describe();
 		}
-  }
-
+ }
+way_out:
+  if (usertree) {
+    for (j = 1; j <= maxuser; j++) free(fsteps[j - 1]);
+    free(fsteps);
+    }
   free(temp->numsteps);
   free(temp->base);
   free(temp);
   free(temp1->numsteps);
   free(temp1->base);
   free(temp1);
-
-	return treefile;
+  return treefile;
 }  /* maketree */
 
 
@@ -1104,18 +1115,22 @@ Static char *maketree(void)
 /************************ MAIN *************************/
 /*******************************************************/
 
-char *dnapars(char** seq, char** seqname, int notu, int *steps, char* toevaluate, int arg_maxtrees,
-	int *bt_weights)
+char* dnapars(char** seq, char** seqname, int notu, int *steps, char* toevaluate, int arg_maxtrees,
+		    int *bt_weights)
 {
-	char* treefile;
+char* treefile;
 int ll;
-	maxtrees = arg_maxtrees;
+maxtrees = arg_maxtrees;
 if (toevaluate){ usertree=true; infile=toevaluate; }
 else usertree=false;
 
 spp=notu;
-if (seq) chars=(int)strlen(seq[0]);
-else chars=0;
+if (seq) {
+  ll = strlen(seq[0]);
+  if (ll >= SHRT_MAX) return NULL;
+  chars = (short)ll;
+  }
+else return NULL;
 nonodes = spp * 2 - 1;
   doinit();
 
@@ -1158,6 +1173,7 @@ for(i=0;i<spp;i++){
       free(y[i]);
       free(nayme[i]);
     }
+    free(y);
     free(enterorder);
     free(names_d);
     free(place_d);
@@ -1178,6 +1194,7 @@ for(i=0;i<spp;i++){
 	p = p->next;
       }
     }
+    free(treenode);
     free(threshwt);
 
 if (steps) *steps = (int)(like_d/(-10.) + .5);
