@@ -33,6 +33,8 @@
  */
 #include "phyml_util.h"
 
+extern int tree_build_interrupted;
+
 /*********************************************************/
 
 void Warn_And_Exit(char *s)
@@ -132,6 +134,9 @@ matrix *JC69_Dist(allseq *data, model *mod)
 	
 	Fors(site,data->c_seq[0]->len,mod->stepsize)
     {
+      if (tree_build_interrupted) {
+	break;
+	}
 		For(j,data->n_otu-1)
 		{
 			for(k=j+1;k<data->n_otu;k++)
