@@ -20,6 +20,8 @@
 #include <math.h>
 #include <ctype.h>
 #include <limits.h>
+#include <setjmp.h>
+static jmp_buf lngjmp_env;
 
 #define Malloc(x) mymalloc(x)
 
@@ -36,7 +38,7 @@ typedef unsigned char boolean;
 int nmlngth;                 /* max number of characters in species name    */
 #define maxuser         2   /* maximum number of user-defined trees    */
 
-
+extern int tree_build_interrupted;
 
 typedef long *steptr;
 typedef enum {
@@ -530,6 +532,9 @@ static void evaluate(node* r)
   long i, steps, term;
   double sum;
 
+  if (tree_build_interrupted) {
+    longjmp(lngjmp_env, 0);
+  }
   sum = 0.0;
   for (i = 0; i < (chars); i++) {
     steps = r->numsteps[i];
@@ -998,6 +1003,9 @@ static char *maketree(void)
 
 	treefile = (char *)malloc(100);
 	treefile[0] = 0;
+  
+  if (setjmp(lngjmp_env)) goto way_out;
+
   if (!usertree) {
     for (i = 1; i <= (spp); i++)
       enterorder[i - 1] = i;
@@ -1076,10 +1084,13 @@ static char *maketree(void)
 		evaluate(root);
 		treefile = describe();
 		}
+  }
+way_out:
+  if (usertree) {
     for(i=0;i<maxuser;i++) free(fsteps[i]);
     free(fsteps);
-  }
-    return treefile;
+    }
+  return treefile;
 }  /* maketree */
 
 
@@ -1142,6 +1153,7 @@ for(i=0;i<nonodes;i++) {
 for(i=0;i<maxtrees;i++) free(bestrees[i]);
 free(bestrees);
 for(i=0;i<spp;i++) free(nayme[i]);
+  free(nayme);
 free(enterorder); free(names); free(place); free(weight); free(threshwt);
 free(treenode);
 free(temp->seq); free(temp1->seq);
