@@ -193,14 +193,14 @@ if(win == NULL) {
 	x = win->w() - w - 5;
 	y = movekeys->y() + movekeys->h() + 20;
 	
-	pdfformat = new Fl_Choice(x,y,w/2,h, PDF_OR_PS" page");
+	pdfformat = new Fl_Choice(x,y,w/2,h, PDF_OR_PS_EXT" page");
 	pdfformat->align(FL_ALIGN_TOP_LEFT);
 	pdfformat->add("A4|LETTER");
 	pdfformat->selection_color(pdfformat->color());
 	pdfformat->labelsize(10);
 	pdfformat->textsize(10);
 	
-	pdffontsize = new Fl_Value_Input(x+w/2,y,w/2,h, PDF_OR_PS" fontsize");
+	pdffontsize = new Fl_Value_Input(x+w/2,y,w/2,h, PDF_OR_PS_EXT" fontsize");
 	pdffontsize->align(FL_ALIGN_TOP_RIGHT);
 	pdffontsize->labelsize(10);
   
