@@ -313,9 +313,15 @@ if(new_n != NULL) view->seqname = new_n;
 
 if(new_s == NULL || new_n == NULL) goto nomem;
 if(lenseq > view->max_seq_length) {
-	lenseq = view->max_seq_length;
-	newseq[lenseq] = 0;
-	fl_alert("Warning: sequence was truncated to current max length %d", lenseq);
+  view->max_seq_length = FL_max(lenseq * 1.3, lenseq + 500);
+  allonge_seqs(view->sequence, view->tot_seqs, view->max_seq_length, view->each_length, 
+	       view->tot_comment_lines, view->comment_line);
+  view->region_line = (char*)realloc(view->region_line, view->max_seq_length + 1);
+  for (int i = 0; i < view->tot_seqs; i++) free(view->col_rank[i]);
+  free(view->col_rank);
+  view->col_rank = prepcolranks(view->sequence, view->tot_seqs, view->max_seq_length, view->each_length, 
+	       ( view->protein ? get_color_for_aa : get_color_for_base ), 
+	       view->numb_gc, view->allow_lower);
 	}
 if( (view->seqname[num-1] = (char *)malloc(strlen(newname)+1)) == NULL) 
 	goto nomem;
@@ -335,8 +341,7 @@ if(view->tot_seqs == 0) {
 	init_dna_scroller(view, 1, newname, is_a_protein_seq(newseq), NULL);
 	set_seaview_modified(view, TRUE);
 	view->DNA_obj->redraw();
-	Fl_Menu_Item *items = (Fl_Menu_Item *)((Fl_Menu_ *)view->menu_file)->menu();
-	items[SAVE_AS].activate();
+	((Fl_Menu_Item*)view->menu_file + SAVE_AS)->activate();
 	return 1;
 	}
 if( (newsel = (int *)malloc(num*sizeof(int))) == NULL) goto nomem;
@@ -358,12 +363,14 @@ if(view->numb_dnacolors > 1) {
 	if( (newcol = (char **)malloc(num*sizeof(char *))) == NULL) 
 		goto nomem;
 	memcpy(newcol, view->col_rank, (num-1)*sizeof(char *) );
-	newcol[num-1] = * prepcolranks(view->sequence+num-1, 1, 
+  char **tmpcol = prepcolranks(view->sequence+num-1, 1, 
 		view->max_seq_length,
 		view->each_length+num-1, 
 		( view->protein ? get_color_for_aa : get_color_for_base ), 
 		view->numb_gc, view->allow_lower);
-	if(newcol[num - 1] == NULL) out_of_memory();
+	if(tmpcol == NULL) out_of_memory();
+  newcol[num-1] = tmpcol[0];
+  free(tmpcol);
 	free(view->col_rank);
 	view->col_rank = newcol;
 	}
@@ -621,38 +628,46 @@ free(temp);
 void edit_sequence_callback(Fl_Widget *ob, void *data)
 {
 struct editseq *editdata = (struct editseq *)ob->window()->user_data();
-char *p, *newseq, *q, *newcolseq;
+char *p, *newseq, *q, **newcolseq;
 SEA_VIEW *view;
 int num, l;
 
 view = editdata->view;
 num = editdata->seqnum;
 
-newseq = (char *)malloc(view->max_seq_length + 1);
+l = strlen( editdata->input->value() );
+newseq = (char *)malloc(l + 1);
 if(newseq == NULL) return;
 q = newseq;
 p = (char *)editdata->input->value() - 1;
 while(*(++p) !=  0) {
 	if( ! ISSEQCHAR(*p) ) continue;
-	if(q - newseq >= view->max_seq_length) {
-		fl_alert("Warning: sequence was truncated to current max sequence length %d", view->max_seq_length);
-		break;
-		}
 	*(q++) = *p;
 	}
 *q = 0;
 l = strlen(newseq);
+  if (l > view->max_seq_length) {
+    view->max_seq_length = FL_max(l * 1.3, l + 500);
+    allonge_seqs(view->sequence, view->tot_seqs, view->max_seq_length, view->each_length, 
+		 view->tot_comment_lines, view->comment_line);
+    view->region_line = (char*)realloc(view->region_line, view->max_seq_length + 1);
+    for (int i = 0; i < view->tot_seqs; i++) free(view->col_rank[i]);
+    free(view->col_rank);
+    view->col_rank = prepcolranks(view->sequence, view->tot_seqs, view->max_seq_length, view->each_length, 
+				  ( view->protein ? get_color_for_aa : get_color_for_base ), 
+				  view->numb_gc, view->allow_lower);
+  }
 if(view->numb_gc > 1) {
-	newcolseq = * prepcolranks(&newseq, 1, 
+	newcolseq = prepcolranks(&newseq, 1, 
 		view->max_seq_length, &l, 
 		( view->protein ? get_color_for_aa : get_color_for_base ), 
 		view->numb_gc, view->allow_lower);
 	if(newcolseq == NULL) return;
 	free(view->col_rank[num]);
-	view->col_rank[num] = newcolseq;
+	view->col_rank[num] = newcolseq[0];
+	free(newcolseq);
 	}
-free(view->sequence[num]);
-view->sequence[num] = newseq;
+strcpy(view->sequence[num], newseq); free(newseq);
 view->each_length[num] = l;
 set_seaview_modified(view, TRUE);
 if(l > view->seq_length) {
