@@ -27,7 +27,6 @@ typedef unsigned int NSUInteger;
 
 // included functions
 char *mac_fname_to_roman(const char *in);
-void fl_ringbell(int a);
 const char *MG_GetBundleResourcesDir(void);
 void show_apropos(Fl_Widget *, void *unused);
 void MG_apple_inits(void);
@@ -45,15 +44,10 @@ extern char *get_res_value(const char *name, const char *def_value);
 extern int printout_block, printout_fontsize;
 extern int printout_vary, printout_black;
 extern Fl_Paged_Device::Page_Format printout_pageformat;
+extern Fl_Paged_Device::Page_Layout printout_layout;
 
 /* globals */
 
-void fl_ringbell(int a)
-{
-  fl_beep(FL_BEEP_DEFAULT);
-}
-
-
 char *mac_fname_to_roman(const char *in)
 /* passage codage pathname vers codage MacRoman qui semble necessaire pour display ecran 
  */
@@ -69,48 +63,21 @@ char *mac_fname_to_roman(const char *in)
   return (ok ? out : (char *)in);
 }
 
-/*
-pid_t get_desc_process(pid_t ascend)
-//returns the pid of the first descending process from process of pid ascend
-//or 0 if no descendant
-{
-	FILE *pipe;
-	char command[] = "/bin/ps -U $USER -o pid,ppid ";
-	char line[2000], *p;
-	long long pid, ppid;
-	
-	pipe = popen(command, "r");
-	fgets(line, sizeof(line), pipe); // skip first line
-	while(TRUE)	{
-		p = fgets(line, sizeof(line), pipe);
-		pid = 0;
-		if(p == NULL) break;
-		sscanf(line, "%lld%lld", &pid, &ppid);
-		if( (pid_t)ppid == ascend) break;
-	}
-	pclose(pipe);
-	return (pid_t) pid;
-}
-
-
-void kill_last_desc(pid_t ascend)
-//kills the deepest descendant process from process ascend, if any
-{
-	pid_t son, father;
-	
-	father = ascend;
-	do {
-		son = get_desc_process(father);
-		if(son != 0) father = son;
-	}
-	while(son != 0);
-	if(father != ascend) kill(father, SIGKILL);
-}
-*/
 
 const char *MG_GetBundleResourcesDir(void)
 {
-  return [[[NSBundle mainBundle] resourcePath] UTF8String];
+  static char *path = NULL;
+  if (!path) {
+    path = (char*)malloc(FL_PATH_MAX);
+    CFURLRef url = CFBundleCopyResourcesDirectoryURL(CFBundleGetMainBundle());
+    CFURLGetFileSystemRepresentation(url,
+				   true,
+				   (UInt8*)path,
+				   FL_PATH_MAX);
+    CFRelease(url);
+    path = (char*)realloc(path, strlen(path) + 1);
+    }
+  return path;
 }
 
 
@@ -129,8 +96,6 @@ static void label(int x, int y, int w, int h, const char *text, NSView *view)
 char *mac_GetOutputFName_Plus(const char *dfault, const char *message, int use_only_button, const char *directory)
 {
   static char pathname[FL_PATH_MAX];
-  NSAutoreleasePool *localPool;
-  localPool = [[NSAutoreleasePool alloc] init]; 
   NSSavePanel *_panel =  [NSSavePanel savePanel];
   CFStringRef cfs_title;
   cfs_title = CFStringCreateWithCString(NULL,message,kCFStringEncodingUTF8);
@@ -179,7 +144,7 @@ char *mac_GetOutputFName_Plus(const char *dfault, const char *message, int use_o
   if([pdfmatrix selectedRow] != printout_black) [pdfmatrix selectCellAtRow:printout_black column:0];
   [view addSubview:pdfmatrix];
 //paper format  
-  NSRect paperrect = {280, -10, 80, 60 };
+  NSRect paperrect = {280, -10, 80, 70 };
   NSMatrix *papermatrix = [[[NSMatrix alloc] initWithFrame:paperrect mode:NSRadioModeMatrix 
 						 cellClass:[NSButtonCell class]
 					      numberOfRows:2 numberOfColumns:1] autorelease];
@@ -198,8 +163,17 @@ char *mac_GetOutputFName_Plus(const char *dfault, const char *message, int use_o
   int desired_paper_cell = (printout_pageformat == Fl_Paged_Device::A4 ? 0 : 1);
   if ([papermatrix selectedRow] != desired_paper_cell) [papermatrix selectCellAtRow:desired_paper_cell column:0];
   [view addSubview:papermatrix];
-//variable site button  
-  NSRect variablerect = {360, 20, 80, 20 };
+  // landscape button
+  NSRect landscaperect = {280, 10, 80, 20 };
+  NSButton *landscape = [[[NSButton alloc] initWithFrame:landscaperect] autorelease];
+  [landscape setButtonType:NSSwitchButton];
+  [landscape setTitle:@"landscape"];
+  [landscape sizeToFit];
+  [landscape setIntValue:(printout_layout == Fl_Paged_Device::LANDSCAPE)];
+  [landscape setEnabled:YES];
+  [view addSubview:landscape];
+  //variable site button  
+  NSRect variablerect = {360, 30, 80, 20 };
   NSButton *variable = [[[NSButton alloc] initWithFrame:variablerect] autorelease];
   [variable setButtonType:NSSwitchButton];
   [variable setTitle:@"variable sites only"];
@@ -210,7 +184,17 @@ char *mac_GetOutputFName_Plus(const char *dfault, const char *message, int use_o
 //add accessory view to panel
   [_panel setAccessoryView:view];
 //run panel  
-  NSInteger retval = [(NSSavePanel*)_panel runModalForDirectory:dir file:fname];
+  NSWindow *key = [NSApp keyWindow];
+  int retval;
+  if (fl_mac_os_version >= 100600) {
+    if (dir) [_panel setDirectoryURL:[NSURL fileURLWithPath:dir]];
+    if (fname) [_panel setNameFieldStringValue:fname];
+    retval = [_panel runModal];
+  }
+  else {
+    retval = [(id)_panel runModalForDirectory:dir file:fname];
+  }
+  [key makeKeyWindow];
   [preset release];
   [dir release];
   if ( retval == NSOKButton ) {//read accessory view state
@@ -219,9 +203,10 @@ char *mac_GetOutputFName_Plus(const char *dfault, const char *message, int use_o
     printout_black = [pdfmatrix selectedRow];
     printout_pageformat = ([papermatrix selectedRow] == 0 ? Fl_Paged_Device::A4 : Fl_Paged_Device::LETTER);
     printout_vary = [variable intValue];
+    printout_layout = [landscape intValue] ? Fl_Paged_Device::LANDSCAPE : Fl_Paged_Device::PORTRAIT;
     strcpy(pathname, [[[_panel URL] path] UTF8String]);
     }
-  [localPool release];
+  [_panel setAccessoryView:nil];
   if ( retval == NSCancelButton ) return NULL;
   return pathname;
 }
@@ -685,4 +670,9 @@ void Fl_Copy_Surface::complete_copy_pdf_and_pict()
   CGContextRelease(gc);
   free(mem);
 }
+
+void mac_tmp_dir_name(char *buffer, int l)
+{
+  CFStringGetCString((CFStringRef)NSTemporaryDirectory(), buffer, l, kCFStringEncodingUTF8);
+}
 #endif //__APPLE__
