@@ -817,7 +817,7 @@ char *replace_ranks_by_names(char *tree, int ntaxa, char **taxnames)
 int process_block_trees(FILE *in, char ***ptrees, char ***pnames)
 {
 	char **trees, **names, **taxnames;
-	char *p;
+	char *p, c;
 	int maxi = 5, ntaxa = 0;
 	trees = (char **)calloc(maxi, sizeof(char *));
 	names = (char **)calloc(maxi, sizeof(char *));
@@ -835,10 +835,11 @@ int process_block_trees(FILE *in, char ***ptrees, char ***pnames)
 				names = (char **)realloc(names, maxi * sizeof(char *));
 				}
 			names[count] = strdup(next_token(in));
-			next_token(in);//skip =
+			next_token(in); //skip =
+			while ( isspace(c=fgetc(in)) ) ; // skip white space
 			trees[count] = (char *)malloc(500000);
 			p = trees[count];
-			*p++ = *next_token(in);
+			*p++ = c; // first char (can be '[' for tree header)
 			while(TRUE) {
 			  *p = fgetc(in);
 			  if(*p == '\n' || *p == '\r') continue;
