@@ -207,7 +207,13 @@ Fl_Window *create_and_run_pseudoterminal(const char *label, struct listener_data
 #ifdef WIN32
 	//add_fd does not work well under WIN32: Fl::wait() blocks in read(fd when nothing comes in the pipe
 	//so an equivalent is done outside from Fl::wait()
-	int masterfd = _open_osfhandle((long)pipe, _O_RDONLY);
+	int masterfd = _open_osfhandle(
+#ifdef _WIN64
+				       (intptr_t)
+#else
+				       (long)
+#endif
+				       pipe, _O_RDONLY);
 #else
 	Fl::add_fd(masterfd, FL_READ | FL_EXCEPT, ext_prog_listener, data);
 #endif
