@@ -6,7 +6,7 @@ extern "C" {
 #include "raa_acnuc.h"
 }
 
-typedef struct {
+struct runningw_struct {
 	raa_db_access *raa; 
 	SEA_VIEW *view;
 	FILE *infile; 
@@ -14,12 +14,12 @@ typedef struct {
 	int elt;
 	int count, interrupted, use_spec_name, truncated;
 	Fl_Box *runb; 
-} runningw_struct;
-
+};
 
 /* local functions */
 int racnuc_fetch(const char *db_name, const char *name, FILE *list, int use_spec_name, SEA_VIEW *view,
-				 Fl_Window *, Fl_Menu_Item *feature_item, const char *scan_word);
+				 Fl_Window *, Fl_Menu_Item *feature_item, const char *scan_word, 
+		 Fl_Widget *, Fl_Button*);
 int racnuc_fetch_file_or_list(FILE *list, int listrank, 
 							   int use_spec_name, raa_db_access *raa, SEA_VIEW *view);
 static void idle_callback(void *data);
@@ -30,8 +30,8 @@ void acnuc_url_callback(Fl_Widget *ob, void *data);
 void ok_callback(Fl_Widget *ob, void *data);
 void fsel_callback(Fl_Widget *ob, void *data);
 void more_callback(Fl_Widget *ob, void *data);
-raa_db_access *open_acnuc_socket(Fl_Window *);
-static void interrupt_callback(Fl_Widget *ob, void *data);
+void close_cb(Fl_Widget *ob, void *data);
+raa_db_access *open_acnuc_socket(Fl_Widget *, Fl_Button*);
 char *import_one_seq(raa_db_access *raa, const char *id, int use_spec_name, int *plength, 
 	int *ptruncated, SEA_VIEW *view, int try_as_keyword, Fl_Menu_Item *feature_item, const char *scan_word);
 int import_one_keyword(raa_db_access *raa, const char *keyword, int use_spec_name,  
@@ -39,20 +39,22 @@ int import_one_keyword(raa_db_access *raa, const char *keyword, int use_spec_nam
 static int scan_list(raa_db_access *raa, int mylist, const char *scan_word, const char *feature_name);
 
 
+extern void interrupt_callback(Fl_Widget *ob, void *data);
 extern int add_seq_to_align(SEA_VIEW *view, char *newname, char *newseq, int lenseq);
 extern int compute_size_params(SEA_VIEW *view, int force_recompute);
 extern void update_comment_gc(int seqnum, int ncbigc, SEA_VIEW *view);
 extern char *get_res_value(const char *name, const char *def_value);
 extern int set_res_value(const char *name, const char *value);
-extern void hide_window_callback(Fl_Widget *ob, void *data);
 extern int compute_wid_names(SEA_VIEW *view, int totseqs);
+extern int fl_create_thread(void (*f) (void *), void* p, unsigned stack_size_KB);
 extern "C" int get_ncbi_gc_number(int gc);
 
 /* globals */
 static const char def_acnuc_server[] = "pbil.univ-lyon1.fr:5558";
 
 int racnuc_fetch(const char *db_name, const char *name, FILE *listfile, int use_spec_name, 
-				 SEA_VIEW *view, Fl_Window *w, Fl_Menu_Item *feature_item, const char *scan_word)
+				 SEA_VIEW *view, Fl_Window *w, Fl_Menu_Item *feature_item, const char *scan_word,
+		 Fl_Widget *ok, Fl_Button *close_b)
 /*
 return value: 0 iff OK
 */
@@ -60,7 +62,7 @@ return value: 0 iff OK
 raa_db_access *raa; 
 int err, length, truncated = FALSE;
 
-raa = open_acnuc_socket(w);
+raa = open_acnuc_socket(ok, close_b);
 if(raa == NULL) return 1;
 err = raa_opendb(raa, (char *)db_name); 
 if(err) {
@@ -113,8 +115,9 @@ int racnuc_fetch_file_or_list(FILE *listfile, int listrank, int use_spec_name, r
 	Fl_Button *runi = new Fl_Button(130,30, 60, 20, "Interrupt");
 	runi->callback(interrupt_callback, &(runningw_data.interrupted));
 	runw->end();
-    runw->position((Fl::w() - runw->w())/2, (Fl::h() - runw->h())/2);
+	runw->position((Fl::w() - runw->w())/2, (Fl::h() - runw->h())/2);
 	runw->set_modal();
+	runw->callback(interrupt_callback, &(runningw_data.interrupted));
 	runw->show();
 #ifndef MICRO
 	runw->hotspot(runw);
@@ -184,8 +187,13 @@ else {
 	}
 }
 
+void raa_win_cb(Fl_Widget *ob, void *data)
+{
+  Fl_Button *b = (Fl_Button*)data;
+  b->do_callback();
+}
 
-typedef struct {
+struct raa_dialog_struct {
 	SEA_VIEW *view;
 	Fl_Input *seq_name_field;
 	Fl_Choice *dbchoice;
@@ -193,7 +201,9 @@ typedef struct {
 	Fl_Check_Button *use_key;
 	Fl_Choice *key_choice;
 	Fl_Input *scan_field;
-	} raa_dialog_struct;
+	Fl_Return_Button *ok;
+	Fl_Button *close_b;
+};
 
 
 void racnuc_dialog(SEA_VIEW *view)
@@ -224,8 +234,8 @@ if(load_form == NULL) {
 	newx = (int)fl_width(raa_fields.seq_name_field->label()) + 10 + 3;
 	raa_fields.seq_name_field->resize(newx, raa_fields.seq_name_field->y(),
 		x - newx + w, raa_fields.seq_name_field->h() );
-	Fl_Return_Button *ok = new Fl_Return_Button(390, 35, 50, 20 , "OK");
-	ok->callback(ok_callback, &raa_fields);
+	raa_fields.ok = new Fl_Return_Button(390, 35, 50, 20 , "OK");
+	raa_fields.ok->callback(ok_callback, &raa_fields);
 	
 	raa_fields.use_key = new Fl_Check_Button(10, 65, 20, 20, "");
 	raa_fields.use_key->callback(use_key_callback, &raa_fields);
@@ -252,9 +262,9 @@ if(load_form == NULL) {
 	fsel->resize(newx, fsel->y(), (int)fl_width(fsel->label()) + 8, fsel->h() );
 	fsel->callback(fsel_callback, &raa_fields);
 
-	Fl_Button *close = new Fl_Button(390, 160, 55, 20 , "Close");
-	close->callback(hide_window_callback, NULL);
-	close->shortcut(FL_COMMAND | 'w');
+	raa_fields.close_b = new Fl_Button(380, 160, 65, 20 , "Close");
+	raa_fields.close_b->callback(close_cb, NULL);
+	raa_fields.close_b->shortcut(FL_COMMAND | 'w');
 	
 	Fl_Check_Button *more = new Fl_Check_Button(5, 160, 135, 20 , "More Databases");
 	more->callback(more_callback, &raa_fields);
@@ -264,10 +274,11 @@ if(load_form == NULL) {
 	acnuc_url_button->callback(acnuc_url_callback, NULL);
 	
 	load_form->end();
-    load_form->position((Fl::w() - load_form->w())/2, 
+	load_form->position((Fl::w() - load_form->w())/2, 
 		(Fl::h() - load_form->h())/2);
 	load_form->set_modal();
 	load_form->hotspot(load_form);
+	load_form->callback(raa_win_cb, raa_fields.close_b);
 	}
 raa_fields.seq_name_field->value("");
 raa_fields.seq_name_field->take_focus();
@@ -285,7 +296,7 @@ void use_key_callback(Fl_Widget *ob, void *data)
 	raa_fields->key_choice->clear();
 	raa_fields->scan_field->value(NULL);
 	if(((Fl_Button *)ob)->value() ) {
-		raa = open_acnuc_socket(ob->window());
+		raa = open_acnuc_socket(raa_fields->ok, raa_fields->close_b);
 		if(raa == NULL || raa_opendb(raa, (char *)raa_fields->dbchoice->text()) || raa->swissprot) {
 			((Fl_Button *)ob)->value(0);
 			if(raa) raa_acnucclose(raa);
@@ -352,7 +363,8 @@ void ok_callback(Fl_Widget *ob, void *data)
 	}
 	else feature_item = NULL;
 	racnuc_fetch(raa_fields->dbchoice->text(), raa_fields->seq_name_field->value(), NULL, 
-		use_spec_name, raa_fields->view, raa_fields->dbchoice->window(), feature_item, scan_word);
+		use_spec_name, raa_fields->view, raa_fields->dbchoice->window(), feature_item, scan_word, 
+		     raa_fields->ok, raa_fields->close_b);
 }
 
 
@@ -361,7 +373,6 @@ void fsel_callback(Fl_Widget *ob, void *data)
 #define MESSAGE "Select file of list of seq. names"
 	raa_dialog_struct *raa_fields = (raa_dialog_struct *)data;
 
-	ob->window()->hide();
 	Fl_Native_File_Chooser *chooser = new Fl_Native_File_Chooser();
 	chooser->type(Fl_Native_File_Chooser::BROWSE_FILE);   // let user browse a single file
 	chooser->title(MESSAGE);                        
@@ -371,7 +382,7 @@ void fsel_callback(Fl_Widget *ob, void *data)
 	int use_spec_name = raa_fields->namingchoice->value() == 1;
 	if(list != NULL) {
 		racnuc_fetch(raa_fields->dbchoice->text(), NULL, list, use_spec_name, raa_fields->view,
-					 raa_fields->dbchoice->window(), 0, NULL);
+					 raa_fields->dbchoice->window(), 0, NULL, raa_fields->ok, raa_fields->close_b);
 		}
 }
 
@@ -385,8 +396,11 @@ int num, tot;
 Fl_Check_Button *b = (Fl_Check_Button *)ob;
 
 if(b->value()) {
-	raa = open_acnuc_socket(ob->window());
-	if(raa == NULL) return;
+	raa = open_acnuc_socket(raa_fields->ok, raa_fields->close_b);
+	if(raa == NULL) {
+	  b->value(0);
+	  return;
+	  }
 	b->deactivate();
 	tot = raa_knowndbs(raa, &names, &descriptions); 
 	for(num = 0; num < tot; num++) {
@@ -396,29 +410,98 @@ if(b->value()) {
 	}
 }
 
+struct intr_struct {
+  char *server_ip;
+  int port;
+  raa_db_access *raa;
+  int err;
+  bool killed;
+  bool done;
+};
 
-raa_db_access *open_acnuc_socket(Fl_Window *w)
-//returns NULL iff error
+void close_cb(Fl_Widget *ob, void *data)
 {
-	raa_db_access *raa = NULL;
-	char *server_ip;
-	int port, err;
-	char *url = get_res_value("racnuc", def_acnuc_server);
-	err = raa_decode_address(url, &server_ip, &port, NULL);
-	if(!err) {
-		my_watch_cursor(w);
-		err = raa_open_socket(server_ip, port , (char *)"seaview", &raa); 
-		fl_reset_cursor(w);
-		}
-	if(err) fl_alert("Can't connect to remote acnuc server:\n%s", url);
-	return raa;
+  if (data == NULL) {
+    ob->window()->hide();
+  }
+  else {
+    intr_struct *ss = (intr_struct*)data;
+    ss->killed = true;
+    ss->done = true;
+    ob->user_data(NULL);
+  }
+}
+
+static void do_nothing_error_mess_proc(raa_db_access* raa, char* message)
+{
+}
+
+static void open_socket_thread(intr_struct *ss)
+//runs in a background thread
+{
+  raa_error_mess_proc = do_nothing_error_mess_proc;
+  ss->err = raa_open_socket(ss->server_ip, ss->port, (char *)"seaview", &(ss->raa));
+  if (ss->killed) {
+    delete ss;
+    return;
+  }
+  Fl::lock(); 
+  ss->done = true;
+  Fl::awake();
+  Fl::unlock();
+}
+			
+raa_db_access *open_acnuc_socket(Fl_Widget *ok, Fl_Button *close_b)
+// attempt to open remote acnuc socket using interruptible thread
+// returns NULL iff error or interrupted
+{
+  raa_db_access *raa = NULL;
+  char *server_ip;
+  int port, err;
+  char *url = get_res_value("racnuc", def_acnuc_server);
+  err = raa_decode_address(url, &server_ip, &port, NULL);
+  if (!err) {
+    intr_struct *ss = new intr_struct;
+    ss->server_ip = server_ip;
+    ss->port = port;
+    ss->raa = NULL;
+    ss->done = false;
+    ss->killed = false;
+    ss->err = 0;
+    my_watch_cursor(ok->window());
+    close_b->user_data(ss);
+    close_b->label("Interrupt");
+    ok->deactivate(); 
+    if (fl_create_thread((void(*)(void*))open_socket_thread, (void*)ss, 0) ) {
+      open_socket_thread(ss); // run in main thread if child thread creation failed
+      }
+    else {
+      while ( !ss->done ) Fl::wait(); // wait for end/kill of child thread
+      }
+    fl_reset_cursor(ok->window());
+    ok->activate(); 
+    close_b->user_data(NULL);
+    close_b->label("Close");
+    if (ss->killed) {
+      fl_alert("%s", "Connection attempt was cancelled");
+      return NULL;
+      }
+    err = ss->err;
+    raa = ss->raa;
+    delete ss;
+  }
+  if (err) {
+    fl_alert("Can't connect to remote acnuc server:\n%s", url);
+    raa = NULL;
+    }
+  return raa;
 }
 
 
-static void interrupt_callback(Fl_Widget *ob, void *data)
+/*static void interrupt_callback(Fl_Widget *ob, void *data)
 {
 *(int *)data = TRUE;
-}
+}*/
 
 
 char *import_one_seq(raa_db_access *raa, const char *id, int use_spec_name, int *plength, 
@@ -468,11 +551,6 @@ if(feature_item != NULL) {
 	mnemo = raa_getattributes(raa, mnemo, NULL, &length, &frame, &gencode, &acc, &descr, &specname, &seq);
 	}
 gencode = get_ncbi_gc_number(gencode);
-if(view->tot_seqs > 0 && length > view->max_seq_length) {
-	*ptruncated = TRUE;
-	length = view->max_seq_length;
-	seq[length] = 0;
-	}
 if(frame == 1) {
 	seq -= 2; /* this terrible thing is possible after raa_getattributes */
 	length += 2;
