@@ -52,9 +52,7 @@ int parse_species_sets_from_header(char *header, int **list_species_sets,
 extern void adjust_menu_edit_modes(SEA_VIEW *view);
 int compute_size_params(SEA_VIEW *view, int force_recompute);
 extern char *create_tmp_filename(void);
-extern int save_fasta_file(const char *fname, char **seq, char **comments,
-			   char **seqname, int totseqs, int *eachlength, region *region_used,
-			   int *sel_seqs, int tot_sel_seqs, int spaces_in_names);
+extern void delete_tmp_filename(const char *base_fname);
 
 
 region *duplicate_region(region *source, const char *name)
@@ -263,11 +261,19 @@ int create_gblocks_mask(SEA_VIEW *view, region *maregion)
   if(p == NULL) return 1;
   strcpy(base_fname, p);
   sprintf(fname, "%s.fst", base_fname);
+  char **newnames = new char*[view->tot_seqs];
+  for (int i = 0; i < view->tot_seqs; i++) {
+    newnames[i] = new char[15];
+    sprintf(newnames[i], "Seq_%d", i);
+    }
   status = save_fasta_file(fname, view->sequence, NULL,
-		      view->seqname, view->tot_seqs, view->each_length, NULL,
-			   NULL, 0, FALSE);
+		      newnames, view->tot_seqs, view->each_length, NULL,
+			   NULL, 0, FALSE, TRUE);
+  for (int i = 0; i < view->tot_seqs; i++) delete newnames[i];
+  delete[] newnames;
   if (status) {
     fl_alert("Cannot write to temporary file %s", fname);
+    delete_tmp_filename(base_fname);
     return 1;
     }
   sprintf(cmd, "\"%s\" %s -p=s -t=%c", q, fname, view->protein?'p':'d');
@@ -342,12 +348,7 @@ int create_gblocks_mask(SEA_VIEW *view, region *maregion)
     free(maregion);
     }
   if (in) fclose(in);
-#ifdef WIN32
-  sprintf(fname, "DEL %s.*", base_fname);
-#else
-  sprintf(fname, "PATH=/bin:/usr/bin rm %s.*", base_fname);
-#endif
-  system(fname);
+  delete_tmp_filename(base_fname);
   return status;
 }
 
@@ -510,7 +511,7 @@ void hide_region_line(SEA_VIEW *view)
 free_region(view->active_region);
 view->active_region = NULL;
 Fl_Menu_Item *items = (Fl_Menu_Item *)view->menu_file;
-items[SAVE_REGIONS].deactivate();
+if (view->tot_sel_seqs == 0) items[SAVE_REGIONS].deactivate();
 compute_size_params(view, TRUE);
 }
 
@@ -1380,7 +1381,7 @@ fin_new_header = new_header;
 *new_header = 0;
 numset = -1;
 while (*header!= 0) {
-	if(strncmp(header,";;@",3) == 0) {
+	if(strncmp(header,";;@ of species =", 16) == 0) {
 		numset++;
 		p=strchr(header,'=') + 1;
 		while(*p == ' ') p++;
