@@ -30,9 +30,11 @@
 #endif
 
 #ifdef NO_PDF
-#define PDF_OR_PS "ps"
+#define PDF_OR_PS "PostScript"
+#define PDF_OR_PS_EXT "ps"
 #else
-#define PDF_OR_PS "pdf"
+#define PDF_OR_PS "PDF"
+#define PDF_OR_PS_EXT "pdf"
 #endif
 
 #define fgets(a,b,c) my_fgets(a,b,c)
@@ -55,12 +57,9 @@ extern LPCWSTR utf8towchar(const char *in);
 extern void my_bell(void); 
 #define fopen(a,b) _wfopen(utf8towchar(a), L##b)
 #define memccpy _memccpy
-#define fl_ringbell(a) my_bell()
 #elif defined __APPLE__
-extern void fl_ringbell(int);
 #else
 #include "FL/x.H"
-#define fl_ringbell(a) XBell(fl_display, a)
 #endif
 #define FL_max(a,b)      ( (a) > (b) ? (a):(b) )
 #define FL_min(a,b)      ( (a) < (b) ? (a):(b) )
@@ -118,7 +117,7 @@ typedef enum {MASE_FORMAT, PHYLIP_FORMAT, CLUSTAL_FORMAT, MSF_FORMAT,
 
 typedef enum {OPEN_ANY, OPEN_MASE, OPEN_PHYLIP, OPEN_CLUSTAL, OPEN_MSF, OPEN_FASTA,
 	OPEN_NEXUS, ACNUC_IMPORT, SAVE, SAVE_AS, SAVE_REGIONS, SAVE_PROT_ALIGN, PRINTOUT, 
-#ifdef WIN32
+#if !defined(__APPLE__)
 	PDFOPTIONS,
 #endif
 	CONCATENATE, NEW_WINDOW, CLOSE_WINDOW,QUIT
@@ -353,6 +352,9 @@ extern int delete_in_all_comments(int numdels, int pos,  SEA_VIEW *view);
 extern char* fl_file_chooser(const char* message, const char* pat, const char* fname, int);
 extern char *run_and_close_native_file_chooser(Fl_Native_File_Chooser *chooser, int keepalive=0);
 extern void trees_callback(Fl_Widget *ob, void *data);
+extern int save_fasta_file(const char *fname, char **seq, char **comments,
+		    char **seqname, int totseqs, int *eachlength, region *region_used,
+		    int *sel_seqs, int tot_sel_seqs, int spaces_in_names, int pad_to_max_length=FALSE);
 #ifdef __APPLE__
 extern char *mac_fname_to_roman(const char *in); 
 #endif
