@@ -207,7 +207,7 @@ void reg_and_pos(SEA_VIEW *view, int *selected, list_segments* active)
   
   int sel1=0, sel2=0, sel3=0, i /*, cur_reg_deb=0*/;
   int lgseq = view->seq_length;
-  list_segments start_end = {0, lgseq-1, NULL};
+  list_segments start_end = {1, lgseq, NULL};
   if (active == NULL) active = &start_end;
   
   /*if(view->protein){
@@ -222,7 +222,7 @@ void reg_and_pos(SEA_VIEW *view, int *selected, list_segments* active)
 
   if(sel1 && sel2 && sel3){
     while(active) {
-      for(i=active->debut; i<=active->fin; i++) selected[i]=1;
+      for(i = active->debut - 1; i < active->fin; i++) selected[i]=1;
       active = active->next;
       }
   }
@@ -359,7 +359,7 @@ static void save_stat_cb(Fl_Widget *ob, void *data)
   fnfc.type(Fl_Native_File_Chooser::BROWSE_SAVE_FILE);
   fnfc.filter("Text\t*.txt\n");
   fnfc.preset_file("statistics.txt");
-  fnfc.options(Fl_Native_File_Chooser::SAVEAS_CONFIRM);
+  fnfc.options(Fl_Native_File_Chooser::SAVEAS_CONFIRM | fnfc.options());
   if ( fnfc.show() != 0 ) return;
   FILE *out = fopen(fnfc.filename(), "w");
   fprintf(out, "Alignment: %s\n\n", (char*)ml->user_data());
