@@ -143,7 +143,7 @@ void svg_tree_save(FD_nj_plot *fd_nj_plot)
   char *p = strrchr(tmp, '.'); if(p != NULL) *p = 0;
   strcat(tmp, ".svg");
   chooser->preset_file(tmp);
-  chooser->options(Fl_Native_File_Chooser::SAVEAS_CONFIRM);
+  chooser->options(Fl_Native_File_Chooser::SAVEAS_CONFIRM | chooser->options());
   char *plotfilename = run_and_close_native_file_chooser(chooser);
   if(plotfilename == NULL) return;
   FILE *out;
