@@ -77,6 +77,7 @@ extern const char *make_binary_or_unrooted(char *arbre);
 extern int save_resources(void);
 extern int set_res_value(const char *name, const char *value);
 extern char *create_tmp_filename(void);
+extern void delete_tmp_filename(const char *base_fname);
 extern int is_view_valid(SEA_VIEW *view);
 char *parsimony(char** seq, char** seqname, int notu, int* steps, char* toevaluate, int arg_maxtrees, int *bt_weights, bool);
 extern int fl_create_thread(void (*f) (void *), void* p, unsigned);
@@ -465,26 +466,30 @@ void run_distance_method(SEA_VIEW *view, int distkind, int remove_all_gaps, int
 	sprintf(tree, "[%s] %s", tree_label, display_tree);
 	free(display_tree);
 	box->window()->hide();//necessary under X11 so tree is in foreground
-	treedraw(tree, view, use_bionj? "BioNJ_tree" : "NJ_tree", FALSE);
+  const char *choice = use_bionj? "BioNJ_tree" : "NJ_tree";
+  const char *n = extract_filename(view->masename);
+  char *title = NULL;
+  if (n) {
+    char *q = (char*)strchr(n, '.');
+    if (q) *q = 0;
+    title = new char[strlen(n) + 12];
+    sprintf(title, "%s-%s", n, choice);
+    if (q) *q = '.';
+    }
+  treedraw(tree, view, n ? title:choice, FALSE);
+  delete[] title;
 }
 
 void savedistinfile(matrix *mat, const char *dirname)
 {
 	FILE *out;
 	int i, j;
-	char fname[300] = "";
 	Fl_Native_File_Chooser *chooser = new Fl_Native_File_Chooser();
 	chooser->type(Fl_Native_File_Chooser::BROWSE_SAVE_FILE);
-  chooser->options(Fl_Native_File_Chooser::SAVEAS_CONFIRM);
+  chooser->options(Fl_Native_File_Chooser::SAVEAS_CONFIRM | chooser->options());
 	chooser->title("Set distance output filename");  
-#ifdef WIN32
-	char separ = '\\';
-#else
-	char separ = '/';
-#endif
-	if(*dirname != 0) sprintf(fname, "%s%c", dirname, separ);
-	strcat(fname, "seqs.dist");
-	chooser->preset_file(fname);
+  chooser->directory(dirname);
+  chooser->preset_file("seqs.dist");
 	char *outfname = run_and_close_native_file_chooser(chooser);
 	if(outfname == NULL) return;
 	out = fopen(outfname, "w");
@@ -997,6 +1002,10 @@ void run_phyml_callback(Fl_Widget *ob, void *data)
     }
   sprintf(input, "%s.phy_phyml_stats.txt", base_fname);
   in = fopen(input, "r");
+  if (in == NULL) {
+    sprintf(input, "%s.phy_phyml_stat.txt", base_fname);
+    in = fopen(input, "r");
+    }
   logL = 0;
   if(in != NULL) {//search tree likelihood
 	  while(TRUE) {
@@ -1004,6 +1013,7 @@ void run_phyml_callback(Fl_Widget *ob, void *data)
 		  p = fgets(line, sizeof(line), in);
 		  if(p == NULL) break;
 		  if( (p=strstr(line, "Log-likelihood:")) != NULL) sscanf(p+15, "%f", &logL);
+		  else if( (p=strstr(line, "Log-likelihood :")) != NULL) sscanf(p+16, "%f", &logL);
 		  }
 	  fclose(in);
 	  remove(input);
@@ -1040,8 +1050,20 @@ void run_phyml_callback(Fl_Widget *ob, void *data)
 	  char *tree = (char *)malloc(strlen(tree_label) + strlen(display_tree) + 4);
 	  sprintf(tree, "[%s] %s", tree_label, display_tree);
 	  free(display_tree);
-	  if (is_view_valid(view)) treedraw(tree, view, "PhyML_tree", FALSE);
+	  if (is_view_valid(view)) {
+	    const char *n = extract_filename(view->masename);
+	    char *title = NULL;
+	    if (n) {
+	      char *q = (char*)strchr(n, '.');
+	      if (q) *q = 0;
+	      title = new char[strlen(n) + 12];
+	      sprintf(title, "%s-%s", n, "PhyML_tree");
+	      if (q) *q = '.';
+	    }
+	    treedraw(tree, view, n ? title:"PhyML_tree", FALSE);
+	    delete[] title;
 	  }
+	}
   remove(input);
   Fl::delete_widget(ob->window());
 way_out:
@@ -1049,6 +1071,7 @@ way_out:
     if(truenames[i]) free(truenames[i]);
     }
   free(truenames);
+  delete_tmp_filename(base_fname);
 }
 
 
@@ -1810,8 +1833,18 @@ way_out:
 	free(pars_names);
 	if (tree != NULL) {
 	  free(tree);
-	  treedraw(final_tree, view, progname, FALSE);
+	  const char *n = extract_filename(view->masename);
+	  char *title = NULL;
+	  if (n) {
+	    char *q = (char*)strchr(n, '.');
+	    if (q) *q = 0;
+	    title = new char[strlen(n) + strlen(progname) + 2];
+	    sprintf(title, "%s-%s", n, progname);
+	    if (q) *q = '.';
 	  }
+	  treedraw(final_tree, view, n ? title:progname, FALSE);
+	  delete[] title;
+	}
 }
 
 
