@@ -641,12 +641,9 @@ hit try_angle_draw(float w, int *px, int *py, float *pdegrees /* clockwise degre
 }
 
 
-#if defined(MICRO) || USE_XFT
-
 hit draw_name_angle(cp_point *phys_pos, const char *nom, FD_unrooted *fd_unrooted, double radians,
 					int doit, int char_width, int char_height, int descend, int color,
 					int ascent)
-//uses the rotated text function available except when X11 and USE_XFT == 0
 {	
 	int x, y;
 	float degrees = - radians * 180/M_PI;
@@ -662,99 +659,6 @@ hit draw_name_angle(cp_point *phys_pos, const char *nom, FD_unrooted *fd_unroote
 	return NO_HIT;
 }
 
-#else
-
-hit draw_letter(point *position, char lettre, FD_unrooted *ob, int doit, int char_width, int ascent);
-
-hit draw_name_angle(cp_point *phys_pos, const char *nom, FD_unrooted *ob, double angle,
-        int doit, int char_width, int char_height, int descend, int color, int ascent)
-//simulates rotated text with unrotated characters
-{
-double x, y, x_delta, y_delta;
-point new_position;
-int stepx;
-hit result;
-static char lettre[2] = "A";
-
-x = phys_pos->x;
-y = phys_pos->y;
-if((angle>= 0 && angle <= M_PI/4) || (angle >= 7*M_PI/4 && angle <= 2*M_PI)) {
-        stepx = TRUE;
-        }
-else if((angle > M_PI/4 && angle < M_PI/2) || 
-                (angle > 3*M_PI/2 && angle < 7*M_PI/4)) {
-        stepx = FALSE;
-        }
-else if((angle >= M_PI/2 && angle <= 3*M_PI/4) || 
-                (angle >= 5*M_PI/4 && angle <= 3*M_PI/2)) {
-        stepx = FALSE;
-        y_delta = char_height;
-        y_delta -= descend;        
-        y_delta *= ( (int) strlen(nom) );
-		if(angle >= M_PI/2 && angle <= 3*M_PI/4) y -= y_delta;
-        else    y += y_delta;
-        x_delta = y_delta / fabs( tan(angle) );
-        x -= x_delta;
-        angle -= M_PI;
-        if(angle < 0) angle += 2*M_PI;
-        }
-else    {
-        stepx = TRUE;
-        x_delta = fl_width(nom);
-        y_delta = x_delta * tan(angle);
-        x -= x_delta;
-        y += y_delta;
-        angle -= M_PI;
-        if(angle < 0) angle += 2*M_PI;
-        }
-if(doit && color) fl_color(FL_RED);
-do      {
-        new_position.x = (int)(x + 0.5);
-        new_position.y = (int)(y + 0.5);
-        result = draw_letter(&new_position, *nom, ob, doit, char_width, ascent);
-        if( result != NO_HIT ) 
-                return result;
-        if(stepx) {
-                lettre[0] = nom[0];
-                x_delta = fl_width(lettre);
-                x += x_delta;
-                y_delta = x_delta * tan(angle);
-                y -= y_delta;
-                }
-        else    {
-                y_delta = char_height;
-                y_delta -= descend;
-                x_delta = y_delta / fabs( tan(angle) );
-                x += x_delta;
-                if(angle <= M_PI/2) y -= y_delta;
-                else    y += y_delta;
-                }
-        nom++;
-        }
-while(*nom != 0);
-if(doit && color) fl_color(FL_BLACK);
-return NO_HIT;
-}
-
-
-hit draw_letter(point *position, char lettre, FD_unrooted *ob, int doit, 
-        int char_width, int ascent)
-{
-static char chaine[2] = " ";
-// clipping
-if(!doit) {
-	if( position->x <= ob->x ) return LEFT_HIT;
-	if( position->x >= ob->x + ob->w ) return RIGHT_HIT;
-	if( position->y - ascent < ob->y )  return TOP_HIT;
-	if( position->y >= ob->y + ob->h ) return BOTTOM_HIT;
-	}
-chaine[0] = lettre;
-if(doit && lettre != ' ') my_draw_text(FL_ALIGN_LEFT, position->x, position->y,
-                ob->font_size, ob->current_font, chaine);
-return NO_HIT;
-}
-#endif
-
 
 void draw_scale(FD_unrooted *fd_unrooted, int charheight, int ascent)
 {
