@@ -21,7 +21,7 @@ int read_msf_align(const char *fname, char ***pseq, char ***pseqname,
 int is_a_protein_seq(char *seq);
 int save_fasta_file(const char *fname, char **seq, char **comments,
 	char **seqname, int totseqs, int *eachlength, region *region_used,
-	int *sel_seqs, int tot_sel_seqs, int spaces_in_names);
+	int *sel_seqs, int tot_sel_seqs, int spaces_in_names, int pad_to_max_length);
 int save_phylip_file(const char *fname, char **seq,
 	char **seqname, int totseqs, int *eachlength, region *region_used, 
 	int *sel_seqs, int tot_sel_seqs, int phylipwidnames);
@@ -59,7 +59,8 @@ known_format what_format(const char *filename);
 char* seaview_file_chooser_save_as(const char* message, const char* fname, SEA_VIEW *view, known_format* new_format);
 const char *extract_dirname(const char *pathname);
 int printout(SEA_VIEW *view, const char *filename, 
-	     int fontsize, int block_size, Fl_Paged_Device::Page_Format pageformat, int vary_only, int ref0, int pdfkindvalue);
+	     int fontsize, int block_size, Fl_Paged_Device::Page_Format pageformat, int vary_only, int ref0, 
+	     int pdfkindvalue, Fl_Paged_Device::Page_Layout layout);
 static void color_pdf_display(SEA_VIEW *view, int (*calc_color_function)( int ), char *oneline, 
 			      int widnames, double x, double y, int fontsize, double char_width, double descender,int num, int current);
 static int calc_vary_lines(int *vary_pos, int widpos);
@@ -247,7 +248,9 @@ void parse_trees_from_header(char *header, SEA_VIEW *view)
 			while(TRUE) {
 				q = strchr(p, '\n');
 				if(strncmp(q+1, ";;", 2) != 0) break;
-				if(*(q + 3) == '$' || *(q + 3) == '@' || *(q + 3) == '#' ) break;
+				if (*(q+3) == '$' && *(q+4) != '\n') break;
+				if (strncmp(q+1, ";;@ of species =", 16) == 0) break;
+				if (strncmp(q+1, ";;# of segments=", 16) == 0) break;
 				p = q + 1;
 				l++;
 				}
@@ -315,7 +318,7 @@ int read_fasta_align(const char *fname, char ***pseq, char ***pseqname,
 {
 FILE *in;
 int totseqs, lseq, l2, l, lenseqs;
-char line[300], *p, *i, c, *q;
+char line[300], *p, *i, c, *q, *r;
 static char ret_message[200];
 char **seq, **seqname, **comments, *tmpseq = NULL;
 
@@ -357,6 +360,7 @@ while( i != NULL ){
 	while(q > line + 1 && (*q == '\n' || *q == '\r')) *(q--) = 0;
 	totseqs++;
 	p = line + 1; 
+  while (*p == ' ') p++;
 	if(spaces_in_names) {
 	  while(*p != '\n') p++;
 	  while(*(p-1) == ' ') p--;
@@ -364,9 +368,11 @@ while( i != NULL ){
 	else {
 	  while(*p != ' ' && *p != '\n') p++;
 	  }
-	l = p - line - 1;
+  r = line + 1;
+  while (*r == ' ') r++;
+	l = p - r;
 	if( (seqname[totseqs] = (char *)malloc(l+1)) == NULL)goto nomem;
-	memcpy(seqname[totseqs], line + 1, l); seqname[totseqs][l] = 0;
+	memcpy(seqname[totseqs], r, l); seqname[totseqs][l] = 0;
 	/* use rest of title line, if any, as comment */
 	while(*p == ' ') p++;
 	l = q - p + 1;
@@ -972,6 +978,10 @@ if(tot_trees > 0) { /* write trees out */
 		fprintf(out, ";;$ %s\n", menu_tree_items[num].label());
 		int l = strlen(trees[num]);
 		for(char *p = trees[num]; p < trees[num] + l; p += 80) {
+			while (*p == '$') { // avoid begin line with ";;$..." that marks a new tree
+			  fputs(";;$\n", out);
+			  p++;
+			  }
 			fprintf(out, ";;%.80s\n", p);
 			}
 		}
@@ -1012,16 +1022,17 @@ return retval;
 
 int save_fasta_file(const char *fname, char **seq, char **comments,
 	char **seqname, int totseqs, int *eachlength, region *region_used,
-	int *sel_seqs, int tot_sel_seqs, int spaces_in_names)
+	int *sel_seqs, int tot_sel_seqs, int spaces_in_names, int pad_to_max_length)
 /* sauver des sequences ou des regions au format fasta
 region_used pointe vers la liste des regions a sauver
 si region_used == NULL, toutes les sequences sont sauvees
+si pad_to_max_length == TRUE && region_used == NULL, les seqs sont allonges a leur longueur max
 rend 0 si ok, 1 si erreur d'ecriture dans le fichier
 	2 si tentative de depasser la longueur d'une sequence
 */
 {
 FILE *out;
-int num, retval = 1, current, ecrit;
+int num, retval = 1, current, ecrit, save_full = (region_used == NULL);
 list_segments *psegment, all_sequence;
 region maregion;
 char *p;
@@ -1042,7 +1053,7 @@ if(region_used == NULL) { /* on veut tout sauver */
 for(num=0; num<totseqs; num++) {
 	if( tot_sel_seqs != 0 && ! sel_seqs[num] ) continue;
 	current = 0; psegment = region_used->list;
-	all_sequence.fin = eachlength[num];
+	if (!save_full || !pad_to_max_length) all_sequence.fin = eachlength[num];
 	fputc('>', out);
 	p = seqname[num];
 	if(spaces_in_names) {
@@ -1215,7 +1226,8 @@ for(i = 0, j = 0; i < totseqs; i++) {
 	gen_check += check_val[j++];
 	}
 gen_check = gen_check % 10000;
-fprintf(out,"PileUp\n\n\n\n   MSF: %d  Type: %c    Check:%6d   .. \n\n",
+  fprintf(out, "!!%2s_MULTIPLE_ALIGNMENT 1.0\n %s", (protein?"AA":"NA"), extract_filename(fname) );
+fprintf(out,"   MSF: %d  Type: %c    Check:%6d   .. \n\n",
 	lenseqs, (protein ? 'P' : 'N'), gen_check);
 for(i = 0 , j = 0; i < totseqs; i++) {
 	if( tot_sel_seqs != 0 && ! sel_seqs[i] ) continue;
@@ -1557,9 +1569,13 @@ Fl_Native_File_Chooser *chooser = new Fl_Native_File_Chooser();
 #endif
 
 chooser->type(Fl_Native_File_Chooser::BROWSE_SAVE_FILE);   
-  chooser->options(Fl_Native_File_Chooser::SAVEAS_CONFIRM);
+chooser->options(Fl_Native_File_Chooser::SAVEAS_CONFIRM | chooser->options());
+#if HAVE_USE_FILTER_EXT || defined(__APPLE__)
+chooser->options(Fl_Native_File_Chooser::USE_FILTER_EXT | chooser->options());
+#endif
 chooser->title(message);  
-chooser->preset_file(fname); 
+chooser->directory(extract_dirname(fname)); 
+chooser->preset_file(extract_filename(fname)); 
 	
 p = types_list;
 for(int f = 0; f < nbr_formats; f++) {
@@ -1594,7 +1610,8 @@ const char *extract_dirname(const char *pathname)
 }
 
 int printout(SEA_VIEW *view, const char *filename, 
-	     int fontsize, int block_size, Fl_Paged_Device::Page_Format pageformat, int vary_only, int ref0, int pdfkindvalue)
+	     int fontsize, int block_size, Fl_Paged_Device::Page_Format pageformat, int vary_only, int ref0, 
+	     int pdfkindvalue, Fl_Paged_Device::Page_Layout layout)
 {
   int num, i, j, k, current, max_seq_length, fin, curr_lines, widnames, 
   res_per_line, nl, firstpage, lines_per_page, use_pdf;
@@ -1615,7 +1632,7 @@ int printout(SEA_VIEW *view, const char *filename,
   use_pdf = (pdfkindvalue != TEXT_ONLY);
   if(use_pdf) {
     surface = new Fl_PDF_or_PS_File_Device();
-    if (surface->begin_document(filename, pageformat)) {
+    if (surface->begin_document(filename, pageformat, layout)) {
       delete surface;
       return 0;
     }
@@ -1661,8 +1678,10 @@ int printout(SEA_VIEW *view, const char *filename,
       else {fputs(fixed, textfile); fputs("\n", textfile);}
       ++curr_lines;
     }
-    sprintf(oneline,"Seaview [blocks=%d fontsize=%d %s] on %s",
-	    block_size, fontsize, pageformat == Fl_Paged_Device::A4 ? "A4" : "LETTER", ctime(&heure));
+    sprintf(oneline,"Seaview [blocks=%d fontsize=%d %s%s] on %s",
+	    block_size, fontsize, pageformat == Fl_Paged_Device::A4 ? "A4" : "LETTER", 
+	    layout == Fl_Paged_Device::LANDSCAPE ? "-landscape" : "",
+	    ctime(&heure));
     if(use_pdf) {
       char *p = strchr(oneline, '\n'); if (p) *p = 0;
       fl_draw(oneline, margin, margin + curr_lines * fontsize); 
