@@ -245,21 +245,21 @@ void set_ncbi_genetic_code(SEA_VIEW *view)
 int gc = 1, num;
 char line[170];
 int status;
-static char codencbinames[18][60] = { "Standard", "Yeast mt", "Vertebrate mt",
+static char codencbinames[19][60] = { "Standard", "Yeast mt", "Vertebrate mt",
 	"Mold+Protozoan+Coelentarate mt, Mycoplasma Spiroplasma", "Invertebrate mt", "Alternative Yeast Nuclear", 
 	"Ciliate+Dasycladacean+Hexamita Nuclear", "Euplotid Nuclear", "Echinoderm+Flatworm mt",
 	"Ascidian mt", "Alternative Flatworm mt", "Blepharisma Macronuclear", 
 	"Bacterial and Plant Plastid", "Chlorophycean mt", "Trematode mt", 
-	"Scenedesmus obliquus mt", "Thraustochytrium mt", "Pterobranchia mt"};
+	"Scenedesmus obliquus mt", "Thraustochytrium mt", "Pterobranchia mt", "Candidate Division SR1"};
 
 if(view->comments == NULL) view->comments = (char **)calloc(view->tot_seqs, sizeof(char *));
 if(view->comments == NULL) return;
-Fl_Window *win = new Fl_Window(750,360);
+Fl_Window *win = new Fl_Window(750,390);
 win->label("Genetic code selection");
 for(num = 0; num < view->tot_seqs; num++)
 		if(view->sel_seqs[num]) break;
 gc = get_ncbi_gc_from_comment(view->comments[num]);
-Fl_Hold_Browser *obj = new Fl_Hold_Browser(5, 25, 740, 310);
+Fl_Hold_Browser *obj = new Fl_Hold_Browser(5, 25, 740, 340);
 obj->textfont(FL_COURIER);
 sprintf(line, "Set genetic code for %s", view->tot_sel_seqs > 1 ? "selected sequences" : view->seqname[num]);
 obj->copy_label(line);
