@@ -82,12 +82,6 @@ char *get_prog_dir(void)
 }
 
 
-void my_bell(void)
-{
-	Beep(0,0);
-} 
-
-
 char **getutf8args(int *pargc)
 {
 	int argc;
