@@ -1,4 +1,5 @@
 #include "matpt.h"
+#include "pdf_or_ps.h"
 #include <stdlib.h>
 #include <stdio.h>
 #include <math.h>
@@ -7,7 +8,6 @@
 
 #ifndef NO_PDF
 #include <pdflib.h>
-static PDF *pdf;
 #endif
 
 
@@ -21,6 +21,7 @@ struct segment {
 	};
 static struct segment segments[MAXSEGMENTS+1];
 gap_site gap_sites[MAX_GAP_SITES];
+extern Fl_Paged_Device::Page_Format printout_pageformat;
 
 /* prototypes */
 void show_dot_plot(char *seq1, char *seq2, char *seqname1, char *seqname2,
@@ -45,8 +46,7 @@ void magnify_reduce_proc(Fl_Widget *ob, void *extra);
 void change_comput_params(Fl_Widget *ob, void *data);
 void compute_proc(Fl_Widget *ob, void *data);
 void interrupt_callback(Fl_Widget *ob, void *data);
-void ps_draw_line(int x1, int y1, int x2, int y2, FD_matpt *matpt,
-	FILE *plotfile);
+void ps_draw_line(int x1, int y1, int x2, int y2, FD_matpt *matpt);
 void ps_set_mat_data(FD_matpt *matpt);
 void plot_button_proc(Fl_Widget *ob, void *data);
 int really_close(FD_matpt *matpt);
@@ -279,7 +279,7 @@ fdui->choice_ref_seq = obj;
 obj = cre_button(0, curr_y, &width, but_height, fontsize, "Interrupt");
 curr_x = align_group->x() + align_group->w() - width;
 obj->position(curr_x, obj->y());
-obj->callback(interrupt_callback, fdui);
+obj->callback(interrupt_callback, &(fdui->interrupted));
 fdui->interrupt_butt = obj;
 
 curr_x = fdui->choice_ref_seq->x() + fdui->choice_ref_seq->w() + 1;
@@ -587,7 +587,7 @@ static int old_w = 0, old_h = 0;
 		set_size_data(matpt);
 		((Fl_Slider *)matpt->region_slider)->value(matpt->region_size);
 		}
-	fl_clip(this->x(), this->y(), this->w(), this->h());
+	fl_push_clip(this->x(), this->y(), this->w(), this->h());
 	if(matpt->rect_only) goto label_draw_rect;
 	fl_overlay_clear();
 /* ecriture du fond */
@@ -775,30 +775,18 @@ fdui->region_slider->redraw();
 
 void interrupt_callback(Fl_Widget *ob, void *data)
 {
-FD_matpt *fdui = (FD_matpt *)data;
-fdui->interrupted = TRUE;
+  *(int *)data = TRUE;
 }
 
 
-void ps_draw_line(int x1, int y1, int x2, int y2, FD_matpt *matpt,
-	FILE *plotfile)
+void ps_draw_line(int x1, int y1, int x2, int y2, FD_matpt *matpt)
 {
-int px, py;
+int px, py, px2, py2;
 px = (int)floor(x1 * matpt->factor + matpt->kx + 0.5);
 py = (int)floor(y1 * matpt->factor + matpt->ky + 0.5);
-#ifdef NO_PDF
-fprintf(plotfile,"%d %d moveto ",px,py);
-#else
-  PDF_moveto(pdf, px, py); 
-#endif
-px = (int)floor(x2 * matpt->factor + matpt->kx + 0.5);
-py = (int)floor(y2 * matpt->factor + matpt->ky + 0.5);
-#ifdef NO_PDF
-fprintf(plotfile,"%d %d lineto stroke\n",px,py);
-#else
-  PDF_lineto(pdf, px, py); 
-  PDF_stroke(pdf);
-#endif
+px2 = (int)floor(x2 * matpt->factor + matpt->kx + 0.5);
+py2 = (int)floor(y2 * matpt->factor + matpt->ky + 0.5);
+  fl_line(px, py, px2, py2);
 }
 
 void ps_set_mat_data(FD_matpt *matpt)
@@ -819,118 +807,73 @@ void plot_button_proc(Fl_Widget *ob, void *data)
 {
 FD_matpt *matpt = (FD_matpt *)data;
 int i;
-char *plotfname, *sp, message[200];
+char message[200];
 int old_phys;
-FILE *plotfile;
-
-  Fl_Native_File_Chooser *chooser = new Fl_Native_File_Chooser();
-  chooser->type(Fl_Native_File_Chooser::BROWSE_SAVE_FILE);
-  chooser->title("Set "PDF_OR_PS" filename");       
-  strcpy(message, matpt->plotname);
-  sp = strchr(message,'.'); 
-  if( sp == NULL ) sp = message + strlen(message);
-  strcpy(sp, "."PDF_OR_PS);
-  chooser->preset_file(message);
-  chooser->filter(
-#ifdef NO_PDF
-		  "PostScript"
-#else
-		  "PDF"
+  
+  Fl_PDF_or_PS_File_Device* surface = new Fl_PDF_or_PS_File_Device();
+  if (surface->start_job(matpt->plotname, printout_pageformat)) return;
+// we use bottom-up coords for PDF and PS output, it's rather tricky.  
+#ifndef NO_PDF
+jmp_buf *jbuf;
+surface->surface_try(&jbuf);
+if (setjmp(*jbuf) == 0) {
+  PDF_set_parameter((PDF*)(surface->pdf()), "topdown", "false"); // set coords bottom-up
 #endif
-		  "\t*."PDF_OR_PS"\n");    
-  chooser->filter_value(0);    
-  plotfname = run_and_close_native_file_chooser(chooser);
-  if(plotfname == NULL) return;
+  surface->start_page();
 #ifdef NO_PDF
-plotfile = fopen(plotfname,"w");
-if(plotfile == NULL) {
-  fl_alert("Cannot write to file\n%s", plotfname);
-  return;
-  }
-fputs("%%!PS-Adobe-3.0\n"
-  "%%Creator: FLTK\n"
-  "%%LanguageLevel: 2\n"
-  "%%Pages: 1\n"
-  "%%BeginFeature: *PageSize A4-Letter\n"
-  "<</PageSize[595 792]>>setpagedevice\n"
-  "%%EndFeature\n"
-  "%%EndComments\n"
-  "%%Page: 1 1\n"
-  "%%PageOrientation: Portrait\n"
-  , plotfile);
-fprintf(plotfile, "1 setlinecap 1 setlinejoin 1 setlinewidth 0 setgray\n");
-fprintf(plotfile,"/basefont /Helvetica findfont 12 scalefont def\n");
-fprintf(plotfile,"basefont setfont\n");
-fprintf(plotfile,"50 200 translate\n");
-fprintf(plotfile,"-10 -10 moveto 510 -10 lineto 510 510 lineto -10 510 lineto \
--10 -10 lineto stroke\n");
+  int w, h;
+  surface->printable_rect(&w, &h);
+  surface->scale(1, -1); // set coords bottom-up
+  surface->origin(30, -h+200);
 #else
-  pdf = PDF_new();
-  if(pdf == NULL) return;
-  if( PDF_begin_document(pdf, plotfname, 0, "compatibility=1.3") == -1) {
-    PDF_delete(pdf);
-    fl_alert("Error opening %s for writing.", plotfname);
-    return;
-  }
-PDF_TRY(pdf) {
-  PDF_set_info(pdf, "Title", plotfname );
-  PDF_set_info(pdf, "Creator", "seaview");
-  int font = PDF_load_font(pdf, "Helvetica", 0, "iso8859-1", "");
-  PDF_begin_page_ext(pdf, 595, 792, "");
-  PDF_setfont(pdf, font, 12);
-  PDF_setlinecap(pdf, 1);
-  PDF_setlinewidth(pdf, 1);
-  PDF_translate(pdf, 50, 200);
-  PDF_moveto(pdf, -10, -10);
-  PDF_lineto(pdf, 510, -10);
-  PDF_lineto(pdf, 510, 510);
-  PDF_lineto(pdf, -10, 510);
-  PDF_lineto(pdf, -10, -10);
-  PDF_stroke(pdf);
+  surface->origin(30, 200);
 #endif
+  fl_color(FL_BLACK);
+  fl_line(-10,-10,510,-10);
+  fl_line(510,-10,510,510);
+  fl_line(-10,-10,-10,510);
+  fl_line(-10,510,510,510);
+  old_phys = matpt->phys_width; matpt->phys_width = 500;
+  ps_set_mat_data(matpt);
 
-old_phys = matpt->phys_width; matpt->phys_width = 500;
-ps_set_mat_data(matpt);
+  ps_draw_line(0, 0, matpt->seqlong1, 0, matpt);
+  ps_draw_line(0, 0, 0, matpt->seqlong2, matpt);
+  for (i = 0; i <= matpt->totsegments; i++)	{
+	  ps_draw_line(segments[i].x, segments[i].y,
+		  segments[i].x+segments[i].longueur,
+		  segments[i].y+segments[i].longueur, matpt);
 
-ps_draw_line(0, 0, matpt->seqlong1, 0, matpt, plotfile);
-ps_draw_line(0, 0, 0, matpt->seqlong2, matpt, plotfile);
-for (i = 0; i <= matpt->totsegments; i++)	{
-	ps_draw_line(segments[i].x, segments[i].y,
-		segments[i].x+segments[i].longueur,
-		segments[i].y+segments[i].longueur, matpt, plotfile);
+	  }
+  matpt->phys_width = old_phys;
+  reset_mat_data(matpt);
 
-	}
-matpt->phys_width = old_phys;
-reset_mat_data(matpt);
-
-sprintf(message,
-	"Horizontal: %.30s (%d)  Vertical: %.30s (%d)  Window: %d Matches: %d",
-	matpt->seqname1, matpt->seqlong1, matpt->seqname2, matpt->seqlong2, 
-		matpt->fenetre, matpt->identites);
+  sprintf(message,
+	  "Horizontal: %.30s (%d)  Vertical: %.30s (%d)  Window: %d Matches: %d",
+	  matpt->seqname1, matpt->seqlong1, matpt->seqname2, matpt->seqlong2, 
+		  matpt->fenetre, matpt->identites);
 #ifdef NO_PDF
-fprintf(plotfile,"-10 -30 moveto (%s) show\n", message);
-fputs("showpage\n%%EOF\n", plotfile);
-fclose(plotfile);
+  surface->scale(1, 1); // back to top-down coords
+  surface->origin(20, h-200+30);
 #else
-  PDF_show_xy(pdf, message, -10, -30);
-  PDF_end_page_ext(pdf, "");
-  PDF_end_document(pdf, "");
-  PDF_delete(pdf);
+  surface->origin(20, 170);
+#endif
+  surface->driver()->font(FL_HELVETICA, 10);
+  fl_draw(message, 0, 0);
+  surface->end_page();
+  surface->end_job();
+#ifndef NO_PDF
 } /* end of PDF_TRY */
-PDF_CATCH(pdf) {
-  if(pdf_catch(pdf)) {
-    fl_alert("Error while writing pdf file:\n"
-	     "[%d] %s: %s\n",
-	     PDF_get_errnum(pdf), PDF_get_apiname(pdf), PDF_get_errmsg(pdf) );
-    PDF_delete(pdf);
+  if( surface->surface_catch() ) {
+    surface->error_catch();
+    delete surface;
     return;
     }
-  }
 #endif
 #ifndef MICRO
   matpt->form->hide(); Fl::flush(); // because of strange bug on 32-bit Linux only
 #endif
-  fl_message("Dot plot is now in file\n%s\nin "PDF_OR_PS" format", plotfname);
+  fl_message("Dot plot is now in file\n%s\nin "PDF_OR_PS" format", surface->outfname());
+  delete surface;
 #ifndef MICRO
   matpt->form->show(); Fl::flush();
 #endif
@@ -1286,8 +1229,11 @@ if(it == NULL) {
 centre = (dmin + dmax)/2; flip = 1;
 for(kk = dmin + 1; kk <= dmax + 1; kk++) {
 	if( Fl::ready() ) {
-		Fl::check();
-		if(matpt->interrupted) break;
+		Fl::wait(0);
+		if (matpt->interrupted) {
+		  do Fl::wait(0); while (Fl::ready());
+		  break;
+		  }
 		if( !matpt->form->visible() ) {
 			*erreur = TRUE;
 			goto fin;
